palette = [
        ('frame_header', 'white', 'dark red', 'bold'),
        ('banner', 'black', 'light gray'),
        ('streak', 'black', 'dark red'),
        ('blue_background', 'black', 'dark blue'),
]

chemical_categories = {
        "Secondary metabolite": (["Alkaloids", "Flavonoids", "Furanocoumarins", "Phenylpropanoids", "Polyketides", "Polyphenols", "Polyynes", "Saponins", "Terpenes_and_terpenoids"], "Metabolites not involved in growth and reproduction", True),
        "Alklaoids": (["Anticholinergic_alkaloids", "Diterpene_alkaloids", "Guanidine_alkaloids", "Halogen-containing_alkaloids", "Indole_alkaloids", "Isoquinoline_alkaloids", "Phenethylamine_alkaloids", "Piperidine_alkaloids", "Pyridine_alkaloids", "Pyrrolidine_alkaloids", "Pyrrolizidine_alkaloids", "Quinoline_alkaloids", "Quinolizidine_alkaloids", "Quinuclidine_alkaloids", "Sesquiterpene_alkaloids", "Steroidal_alkaloids", "Tropane_alkaloids", "Tryptamine_alkaloids", "Xanthines"], "Nitrogenous base(amine)", True),
        "Phenylpropanoids": (["Coumarins", "Coumestans", "Flavonoids", "Phenylpropanoid_glycosides", "Hydroxycinnamic_acids", "Lignans", "Monolignols", "O-Methylated_phenylpropanoids", "Phenylpropanoids_metabolism", "Phenylpropenes", "Phenylpropionate_esters", "Stilbenoids"], "Phenyl + Propene tail", True),
        "Phenethylamines": (["2-Aminoindanes", "2-Benzylpiperidines", "2,5-Dimethoxyphenethylamines", "5-Benzofuranethanamines", "6-Benzofuranethanamines", "Aminotetralins", "Catecholamines", "Cathinones", "Diphenylethylpiperazines", "Phenethylamine_alkaloids", "Phenylethanolamines", "Psychedelic_phenethylamines", "Substituted_amphetamines", "Tetrahydroisoquinolines"], "Phenylethyl amines", True),
        "Apocynaceae alkaloids": (["Iboga alklaoid", "Vinca alkaloids"], "Alkaloids of Apocynaceae", False),
}
drug_categories = {
        "Acetylcholine_receptors": (["Anticholinergics", "Cholinergics"], "Drugs type on acetylcholine", False),
        "Musacarinic_acetylcholine_receptors": (["Musacarininc agonists", "Musacarinic antagonists"], "Musacarinic agonist/antagonist", False),
        "Nicotinic_acetylcholine_receptors": (["Nicotinic agonists", "Nicotinic antagonists"], "Nicotinic agonist/antagonist", False),
        "Adrenergic_receptors": (["Alpha blockers", "Alpha-adrenergic agonists", "Beta blockers", "Beta-adrenergic agonists"], "Adrenergic agonist/antagonist", False),
        "Dopamine_receptors": (["Dopamine agonists", "Dopamine antagonists"], "Dopamine agonist/antagonist", False),
        "GABA_receptors": (["GABA_receptor_agonists", "GABA_receptor_antagonists"], "GABA agonist/antagonist", False),
        "Ionotropic_glutamate_receptors": (["AMPA_receptor_agonists", " AMPA_receptor_antagonists", "Kainate_receptor_agonists", "Kainate receptor_antagonist", "NMDA_receptor_agonists", "NMDA_receptor_antagonists", "NMDA_receptor_modulators"], "Ionotropic glutamate agonist/antagonist", False),
        "Metabotropic_glutamate_receptors": (["Metabotropic glutamate receptor agonists", "Metabotropic glutamate receptor antagonists"], "Metabotropic glutamate receptor agonist/antagonist", False),
        "Histamine_receptors": (["Histamine agonists", "Antihistamines"], "Histamine agonist/antagonist", False),
        "Serotonin_receptors": (["Serotonin agonists", "Serotonin antagonists"], "Serotonin agonist/antagonist", False),
        "Opioid_receptors": (["Delta-opioid agonists", "Delta-opioid antagonists", "Kappa agonists", "Kappa antagonists", "Mu-opioid agonists", "Mu-opioid antagonists", "Noiciceptin receptor agonists", "Noiciceptin receptor antagonists"], "Opioid agonist/antagonist", False),
}
"""Later: "https://en.wikipedia.org/wiki/Category:Monoamine_releasing_agents", "https://en.wikipedia.org/wiki/Category:Reuptake_inhibitors" """ 
stubs = ['Alkaloid-stub']



import urwid
from termcolor import colored
from WikiFiller.lib.Globals import palette

class WikiCategory:
    def __init__(self, title, categories):
        wiki_categories = self.form_categories(title, categories)
        categories = urwid.Pile(wiki_categories)
        top = urwid.Filler(categories, valign='top')
        top = urwid.AttrMap(top, 'blue_background')
        self.loop = urwid.MainLoop(top, palette, unhandled_input=self.key_handler)
        self.loop.run()

    def form_categories(self, title, categories):
        header_text = urwid.AttrMap(urwid.Text(('banner', u"Wikipedia Categories"), align="center"), 'streak')
        pile_list = [urwid.AttrMap(urwid.CheckBox(t), 'banner') for t in categories]
        pile_list = [urwid.Padding(t, align=('relative', 18), width=('relative', 95)) for t in pile_list]
        horizontal_rule = urwid.Divider()
        continue_button = urwid.AttrMap(urwid.Button(('banner', 'Continue')), 'banner')
        continue_button = urwid.Padding(continue_button, align=('relative', 5), width=('relative', 8))
        urwid.connect_signal(continue_button.original_widget.original_widget, 'click', self.on_continue_clicked)
        title_text = urwid.Text(('banner_title', f"{title}:"), align="left")
        pile_list.insert(0, header_text)
        pile_list.insert(1, horizontal_rule)
        pile_list.insert(2, title_text)
        pile_list.insert(3, horizontal_rule)
        pile_list.append(horizontal_rule)
        pile_list.append(continue_button)
        return pile_list

    def on_continue_clicked(self, button):
        from requests import get
        widget_var = self.loop.widget.original_widget.original_widget.widget_list
        checkboxes_list = [widget.original_widget.original_widget for widget in widget_var if isinstance(widget, urwid.decoration.Padding)]
        checkboxes_list = [checkbox for checkbox in checkboxes_list if isinstance(checkbox, urwid.wimp.CheckBox)]
        output = [(c.get_label(), c.get_state()) for c in checkboxes_list]
        self.output = output
        raise urwid.ExitMainLoop()

    def key_handler(self, key):
        if key in ('f8', 'q', 'Q'):
            raise urwid.ExitMainLoop()




from requests import get
from re import search as re_search
from bs4 import BeautifulSoup
from urllib.parse import urlparse
from traceback import print_exc
from string import ascii_uppercase
from termcolor import colored

from WikiFiller.lib.Category import WikiCategory

def chemspider_fetch(name: str) -> str:
    if not name:
        return ""
    try:
        response = get(f'http://www.chemspider.com/Search.aspx?q={name}')
        if not response.history:
            if "0 results" in response.text:
                print(f"Chemspider fetch failed for {name}, {response.history}")
                return ""
            else:
                code = BeautifulSoup(response.text, 'html.parser').find_all('div', {"class":"results-wrapper table"})
                link = code[0].find_all('tbody')[0].find_all('tr')[0].find_all('td')[0].find_all('a')[0]['href']
                chemspider = re_search(r'\d+', urlparse(link).path)
                return chemspider.group() if chemspider else ""
        else:
            code = BeautifulSoup(response.history[0].text, 'html.parser')
            link = code.find_all('a')[0]
            chemspider = urlparse(link['href']).path.split('.html')[0].split('.')[-1]
            return chemspider
    except Exception as E:
        print(E)
        print_exc()

def chebi_fetch(name: str) -> str:
    if not name:
        return ""
    match_string = "Sorry, no results"
    try:
        r = get(f'https://www.ebi.ac.uk/chebi/advancedSearchFT.do?searchString={name}')
        code = r.text
        if match_string in code:
            print("ChEBI fetch failed")
            return ""
        """ List of pages link else direct page """
        if not r.history:
            link = BeautifulSoup(code, 'html.parser').find_all('a', {'target': '_top'})[0]
            chebi = urlparse(link['href']).query.split(':')[-1]
        else:
            chebi = urlparse(r.url).query.split(':')[-1]
        return chebi
    except Exception as E:
        print(E)
        print_exc()

def cas_fetch(name: str) -> str:
    try:
        url = f"https://commonchemistry.cas.org/results?q={name}"
        r = get(url)
        code = r.text
        if not name or '0 results found' in code:
            print("CAS No fetch failed")
            return ""
        link = BeautifulSoup(code, 'html.parser').find_all("div", {"aria-label":True})[0]['aria-label']
        cas = link.split(',')[0].split(':')[-1].strip()
        return cas
    except Exception as E:
        print(E)
        print_exc()

def unii_fetch(name: str) -> str:
    try:
        url = f"https://fdasis.nlm.nih.gov/srs/auto/{name}"
        code = get(url).text
        if not name:
            return ""
        link = BeautifulSoup(code, 'html.parser').find_all("link", {"rel":"canonical"})
        if not link:
            return ""
        unii = urlparse(link[0]['href']).path.split('/')[-1].upper()
        return unii
    except Exception as E:
        print(E)
        print_exc()

def fix_molecular(formula: str) -> list:
    organics = tuple(ascii_uppercase)
    organic_present = [elem for elem in organics if elem in formula]
    start = []
    for elem_i, elem in enumerate(organic_present):
        initial_index = None
        final_index = None
        for letter_i, letter in enumerate(formula):
            if elem == letter:
                initial_index = letter_i
            else:
                if (elem_i + 1) < len(organic_present):
                    if organic_present[int(elem_i + 1)] == letter:
                        final_index = letter_i
                        break
                else:
                    final_index = None
        start.append((elem, initial_index, final_index))
    new_start = []
    for l in start:
        elem, initial, final = l[0], l[1], l[2]
        if final:
            unfixed = formula[initial:final]
        else:
            unfixed = formula[initial:]
        fixed_element, element_atom = elem, unfixed.split(elem)[-1]
        new_start.append((fixed_element, element_atom))
    return(new_start)

def wiki_molecular(element_list: list) -> str:
    l = []
    for elem in element_list:
        elem, number = elem[0], elem[1]
        wiki_like = f"{elem} = {number}"
        l.append(wiki_like)
    return " | ".join(l)

def category_filter(type_categories_list):
    mega_list = []
    for type_categories in type_categories_list:
        for category_key in type_categories.keys():
            sub_category, description, super_category_allowed = type_categories[category_key]
            user = input(f"Does this category {colored(category_key, color='red', attrs=['bold'])} ({description}) fit inside article (Y/N)? ")
            if user.upper() == "Y":
                to_add = WikiCategory(category_key, sub_category).output
                mega_list.extend([cat_tuple[0] for cat_tuple in to_add if cat_tuple[-1]])
                if super_category_allowed:
                    mega_list.append(category_key)
    return "\n".join(f'[[Category:{c}]]' for c in set(mega_list))



from requests import get
from bs4 import BeautifulSoup
from urllib.parse import urlparse
from concurrent.futures import ThreadPoolExecutor
from pubchempy import get_compounds, BadRequestError, TimeoutError

def get_name_from_cid(cid):
    url = f"https://pubchem.ncbi.nlm.nih.gov/compound/{cid}"
    compound_data = BeautifulSoup(get(url).text, 'html.parser')
    title = compound_data.find_all("meta", {"property":"og:title"})
    if title:
        title = title[0].attrs['content']
    else:
        title = ""
    return title

def change_molecular(molecular_list: list, atom_changes: list = []) -> list:
    if not atom_changes:
        changed_list = ["".join(changable) for changable in molecular_list]
        return changed_list

    atom_dict = {elem[0]: elem[1] for elem in molecular_list}
    for atom_change in atom_changes:
        atom_type, atom_name, atom_number = atom_change[0], atom_change[1], atom_change[2]
        if atom_type == "+":
            if atom_name in atom_dict:
                new_atom_number = str(int(atom_dict[atom_name]) + int(atom_number))
                atom_dict[atom_name] = new_atom_number
            else:
                new_atom_number = str(atom_number)
                atom_dict[atom_name] = new_atom_number
        elif atom_type == "_":
            if atom_name in atom_dict:
                new_atom_number = str(int(atom_dict[atom_name]) - int(atom_number))
                if int(new_atom_number) != 0:
                    atom_dict[atom_name] = new_atom_number
                else:
                    atom_dict.pop(atom_name)
    atom_dict = dict(sorted(atom_dict.items()))
    new_list = []
    for a,n in list(atom_dict.items()):
        joined = "".join((a,n)) if int(n) != 1 else a
        new_list.append(joined)
    return new_list

def fix_molecular(formula: str) -> list:
    from string import ascii_uppercase
    organics = tuple(ascii_uppercase)
    organic_present = [elem for elem in organics if elem in formula]
    start = []
    for elem_i, elem in enumerate(organic_present):
        initial_index = None
        final_index = None
        for letter_i, letter in enumerate(formula):
            if elem == letter:
                initial_index = letter_i
            else:
                if (elem_i + 1) < len(organic_present):
                    if organic_present[int(elem_i + 1)] == letter:
                        final_index = letter_i
                        break
                else:
                    final_index = None
        start.append((elem, initial_index, final_index))
    new_start = []
    for l in start:
        elem, initial, final = l[0], l[1], l[2]
        if final:
            unfixed = formula[initial:final]
        else:
            unfixed = formula[initial:]
        fixed_element, element_atom = elem, unfixed.split(elem)[-1]
        new_start.append((fixed_element, element_atom))
    return new_start

def get_smiles_from_nist(url):
    if not url:
        return url
    r = BeautifulSoup(get(url).text, 'html.parser')
    #name = r.find_all('h1', {'id':'Top'})[0].string
    inchi = r.find_all('span', {'clss':'inchi-text'})
    if not inchi:
        return "", ""
    inchi = inchi[0].string
    smiles = get_compounds(inchi, 'inchi')[0].canonical_smiles
    return smiles

def new_molecular_formula_fetch_nist(molecular_formula: str) -> dict:
    if not molecular_formula:
        return {}
    try:
        compounds = get_compounds(molecular_formula, 'formula')
    except BadRequestError:
        return {}
    except TimeoutError:
        return {}
    with ThreadPoolExecutor(max_workers=20) as executor:
        future_objects = [executor.submit(get_name_from_cid, compound.cid) for compound in compounds]
    names = [future_object.result() for future_object in future_objects]
    compounds_all = dict(zip(names, compounds))
    return compounds_all

def molecular_formula_fetch_nist(molecular_formula: str) -> dict:
    url = f"https://webbook.nist.gov/cgi/cbook.cgi?Formula={molecular_formula}&NoIon=on&Units=SI"
    r = BeautifulSoup(get(url).text, 'html.parser')
    if not molecular_formula or "Not found" in r.find_all('title')[0].string:
        return {}
    #compounds = [print(elem, type(elem)) for elem in r.find_all('ol')]
    compounds = [elem.findChildren("a") for elem in r.find_all('ol')][0]
    """ Change into threads later """
    with ThreadPoolExecutor(max_workers=20) as executor:
        future_objects = [executor.submit(get_smiles_from_nist, f"https://webbook.nist.gov{compound['href']}") for compound in compounds]
    smiles = [future_object.result() for future_object in future_objects]
    names = [compound.string for compound in compounds]
    links = dict(zip(names, smiles))
    return links
        #print(elem)
        #print(type(elem.findChildren("a")))
        #breakpoint()
    #breakpoint()
    # url = f"https://webbook.nist.gov/cgi/cbook.cgi?Formula={molecular_formula}&NoIon=on&Units=SI"
    # r = BeautifulSoup(get(url).text, 'html.parser')
    # if not molecular_formula or 'Not Found' in r.find_all('title')[0].string:
        # return []
    # compounds_links = [elem.findChildren("a") for elem in r.find_all('ol')[0]]
    # compounds = [elem.findChildren("a") for elem in r.find_all('ol')][0]
    # compounds_everything = [(get_name_from_cid(compound.cid), compound) for compound in compounds]
    # links = [f"https://webbook.nist.gov{compound['href']}" for compound in compounds]
    # compounds_all = [get_names_inchi_from_nist(link) for link in links]
    # compounds_all = [c for c in compounds_all if c[0] and c[1]]
    # names_smiles = [(c[0], get_compounds(c[1], 'inchi')[0].canonical_smiles) for c in compounds_all]
    # breakpoint()
    # return

# def molecular_formula_fetch_wiki(molecular_formula: str) -> list:
    # url = f'https://en.wikipedia.org/wiki/{molecular_formula}'
    # response = get(url)
    # r = BeautifulSoup(response.text, 'html.parser')
    # if not molecular_formula or 'Wikipedia does not have an article' in response.text:
        # return []
    # if not molecular_formula in r.find_all('title')[0].string:
        # return []
    # compound_li = r.find_all('div', {'class':'mw-parser-output'})[0].find_all('li')
    # titles = [l.find_all('a', {"title":True}) for l in compound_li]
    # final_titles = [t[0]['title'] for t in titles if t]
    # href = [l.find_all('a', {"href":True})[0]['href'].split('/wiki')[-1] for l in compound_li]
    # final_href = [h for h in href if not '#' in h]
#     return ["".join(f) for f in zip(final_titles, final_href)]



#!/usr/bin/python3
from bs4 import BeautifulSoup
from requests import get
from argparse import ArgumentParser
from pubchempy import get_compounds
from fuzzywuzzy import fuzz
from operator import itemgetter
from pyperclip import copy as clipboard_copy

#from WikiFinder.lib.Browser import Browser
from WikiFinder.lib.Functions import change_molecular, fix_molecular
from WikiFinder.lib.Functions import molecular_formula_fetch_nist
#from WikiFinder.lib.Functions import molecular_formula_fetch_wiki

# from lib.Functions import change_molecular, fix_molecular
# from lib.Functions import molecular_formula_fetch

def main():
    parser = ArgumentParser()
    group = parser.add_mutually_exclusive_group()
    group.add_argument('-m', '--molecular-formula', type=str, help="Outputs list of chemicals from molecular formula")
    group.add_argument('-n', '--name', type=str, help="Ouputs list of chemicals matching compound molecular formula from name")
    parser.add_argument('-c', '--change', default=[], nargs='*', help="Enter atom change of compound using spaces. '+' increases atom while '_' decreases. Eg -c '_C1' '+H3'")
    parser.add_argument('-o', '--output', type=str, help="Enter output filename")
    argv = parser.parse_args()

    if not argv.molecular_formula and not argv.name:
        print("Use --help")
        exit()

    #browser_driver = Browser()
    #browser_driver.driver.get(url)

    #def molecular_formula_from_name(name):
        #url = f'https://pubchem.ncbi.nlm.nih.gov/compound/{name}'
        #compound_data = BeautifulSoup(get(url).text, 'html.parser')

        #title = compound_data.find_all("meta", {"property":"og:title"})[0].attrs['content']
        #molecular_formula = change_molecular(fix_molecular(browser_driver.find_element_by_id(By.ID,"Molecular-Formula").upper()), argv.change)
        #return molecular_formula
    compound = {
        'name': '',
        'compound': '',
        'formula': '',
        'smiles': '',
    }
    if argv.name:
        compound['name'] = argv.name
        compound['compound'] = get_compounds(argv.name, 'name')[0]
        compound['smiles'] = compound['compound'].canonical_smiles
        compound['formula'] = change_molecular(fix_molecular(compound['compound'].molecular_formula), argv.change)
        #molecular_formula = molecular_formula_from_name(argv.name)
    else:
        compound['formula'] = change_molecular(fix_molecular(argv.molecular_formula), argv.change)
    compound['formula'] = "".join(compound['formula'])

    compounds_nist = molecular_formula_fetch_nist(compound['formula'])
    #compounds_wiki = molecular_formula_fetch_wiki(molecular_formula)
    #final_compounds = compounds_nist + compounds_wiki
    final_compounds = []
    if compound['smiles']:
        for name, smile in list(compounds_nist.items()):
            n = fuzz.ratio(compound['smiles'], smile)
            final_compounds.append((name, smile, n))
        final_compounds.sort(key = itemgetter(-1), reverse=True)
        final_compounds = [f"{name}:{smile}" for name,smile,n in final_compounds]
    else:
        final_compounds = list(compound_nist.keys())
    compounds = "\n".join(final_compounds)
    if compound['smiles']:
        s = f"Printing compounds with same {compound['formula']} sorted by most similar matching smile {compound['smiles']}"
    else:
        s = f"Printing compounds with same {compound['formula']}"
    print(s)
    print(compounds)

    clipboard_copy(compounds)
    #browser_driver.driver.quit()

    if argv.output:
        with open(argv.output, 'w+') as f:
            f.write(compounds)

if __name__ == '__main__':
    main()



#!/usr/bin/python3
from requests import get
from bs4 import BeautifulSoup
from argparse import ArgumentParser
from pyperclip import copy as clipboard_copy
from pubchempy import get_compounds
from selenium.webdriver.common.by import By

from WikiFiller.lib.Browser import Browser
from WikiFiller.lib.Globals import drug_categories, chemical_categories
from WikiFiller.lib.Template import chembox_part_1, chembox_part_2, chembox_part_3, chembox_part_4
from WikiFiller.lib.Functions import chemspider_fetch, chebi_fetch, cas_fetch, unii_fetch
from WikiFiller.lib.Functions import fix_molecular, category_filter, wiki_molecular

def main():
    parser = ArgumentParser()
    group = parser.add_mutually_exclusive_group()
    group.add_argument('-c', '--chembox', action="store_true", help="Generates chembox")
    group.add_argument('-d', '--drugbox', action="store_true", help="Generates drugbox")
    parser.add_argument('-n', '--name', type=str, help="Enter pubchem compound name from pubchem url")
    parser.add_argument('-C', '--category', choices=['chembox', 'drugbox', 'all'], help="Add category in articles")
    parser.add_argument('-o', '--output', type=str, help="Enter output filename")
    argv = parser.parse_args()

    if not argv.name:
        print("Use --help")
        exit()
    if not argv.chembox and not argv.drugbox:
        print("Use --help")
        exit()

    url = f'https://pubchem.ncbi.nlm.nih.gov/compound/{argv.name}'
    #browser_driver = Browser()
    #browser_driver.driver.get(url)

    compound_data = BeautifulSoup(get(url).text, 'html.parser')
    title = compound_data.find_all("meta", {"property":"og:title"})[0].attrs['content']
    print(f"Creating article template on: {title}")
    compound = get_compounds(argv.name, 'name')[0]
    pubchem_cid = compound.cid
    iupac = compound.iupac_name
    inchi = compound.inchi
    inchi_key = compound.inchikey
    smiles = compound.canonical_smiles
    molecular_formula = fix_molecular(compound.molecular_formula.upper())
    wiki_formula = wiki_moecular(molecular_formula)
    mesh_names = []
    depositor_names = compound.synonyms

    #pubchem_cid = compound_data.find_all("meta", {"property":"og:url"})[0].attrs['content'].split('/')[-1]
    #iupac = browser_driver.find_element_by_id(By.ID,"IUPAC-Name")
    #inchi_full = browser_driver.find_element_by_id(By.ID,"InChI")
    #inchi = inchi_full.split('=')[-1]
    #inchi_key = browser_driver.find_element_by_id(By.ID,"InChI-Key")
    #smiles = browser_driver.find_element_by_id(By.ID,"Canonical-SMILES")
    ec_no = ""  #browser_driver.find_element_by_id(By.ID,"European-Community-(EC)-Number")
    #molecular_formula = fix_molecular(browser_driver.find_element_by_id(By.ID,"Molecular-Formula").upper())
    #wiki_formula = wiki_molecular(molecular_formula)
    #mesh_names = browser_driver.find_element_by_id(By.ID,"MeSH-Entry-Terms", first=False)
    #depositor_names = browser_driver.find_element_by_id(By.ID,"Depositor-Supplied-Synonyms", first=False)

    other_names = [x for x in depositor_names if not any(map(str.isdigit, str(x))) and not '-' in x and not 'pubchem' in x.lower()]
    other_names.extend([x for x in mesh_names if not any(map(str.isdigit, str(x))) and not '-' in x and not 'mesh' in x.lower()])
    other_names = list(set(other_names))
    alt_titles = other_names.copy()
    other_names = ("\n | ".join(other_names)) if other_names else ""
    chembl = ""
    chemspider = chemspider_fetch(argv.name) or chemspider_fetch(title) or chemspider_fetch(smiles) or [holder for alt_title in alt_titles if (holder := chemspider_fetch(alt_title))][0]
    chebi = chebi_fetch(argv.name) or chebi_fetch(title) or chebi_fetch(inchi_full) or [holder for alt_title in alt_titles if (holder := chebi_fetch(alt_title))][0]
    cas_no = cas_fetch(argv.name) or cas_fetch(title) or cas_fetch(inchi_full) or [holder for alt_title in alt_titles if (holder := cas_fetch(alt_title))][0]
    unii = unii_fetch(argv.name) or unii_fetch(title) or unii_fetch(inchi_full) or [holder for alt_titel in alt_titles if (holder := unii_fetch(alt_title))][0]

    #browser_driver.driver.quit()

    if argv.chembox:
        chembox_1 = chembox_part_1 % (iupac, other_names)
        chembox_2 = chembox_part_2 % (cas_no, chemspider, chebi, chembl, ec_no, inchi, inchi_key, smiles, pubchem_cid, unii)
        chembox_3 = chembox_part_3 % (wiki_formula, title)
        if argv.category:
            if argv.category == "all":
                category_list = [chemical_categories, drug_categories]
            elif argv.category == "drugbox":
                category_list = [drug_categories]
            else:
                category_list = [chemical_categories]
            chembox_4 = chembox_part_4 % (category_filter(category_list), "") #to de developed stubs
        else:
            chembox_4 = ""
        full_chembox_template = chembox_1 + chembox_2 + chembox_3 + chembox_4
    elif argv.drugbox:
        print("Not yet developed")

    print("Copying to clipboard")
    clipboard_copy(full_chembox_template)

    if argv.output:
        with open(argv.output, 'w+') as f:
            f.write(full_chembox_template)

if __name__ == '__main__':
    main()



from selenium.webdriver import Firefox
from selenium.common.exceptions import NoSuchElementException

class Browser:
    def __init__(self):
        self.driver = Firefox()
        self.driver.set_page_load_timeout(18)
        self.driver.implicitly_wait(10)

    def find_element_by_id(self, by, value, first=True):
        try:
            data = self.driver.find_element(by, value).text.split('\n')
            if first:
                return data[1]
            else:
                return data
        except NoSuchElementException:
            print(f"No such element: {value}")
            return ""
        except Exception as E:
            print(E)

